/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jbi.messaging.exchange;

import static org.junit.Assert.assertEquals;

import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.junit.After;
import org.junit.Test;
import org.mockito.Mockito;
import org.ow2.petals.jbi.messaging.exchange.impl.DefaultMessageExchangeFactory;

import com.ebmwebsourcing.easycommons.thread.ExecutionContext;

/**
 * @author Olivier Fabre - EBM WebSourcing
 */
public class MessageExchangeFactoryImplTest extends MessageExchangeCheckHelper {

    private ServiceEndpoint consumerEndpointMock = Mockito.mock(ServiceEndpoint.class);

    @After
    public void after() {
        ExecutionContext.getProperties().clear();
    }

    @Test
    public void testBuildUseBuilderConsumerEp() throws MessagingException {
        AbstractMessageExchangeFactory abstractMessageExchangeFactory = (AbstractMessageExchangeFactory) DefaultMessageExchangeFactory
                .create(consumerEndpointMock, null);
        PetalsMessageExchange messageExchangeImpl = abstractMessageExchangeFactory.createExchange();

        assertEquals(consumerEndpointMock, messageExchangeImpl.getConsumerEndpoint());
    }
    
    @Test
    public void testBuildUseBuilderConsumerEpForEndpoint() throws MessagingException {
        ServiceEndpoint targetEndpointMock = Mockito.mock(ServiceEndpoint.class);

        AbstractMessageExchangeFactory abstractMessageExchangeFactory = (AbstractMessageExchangeFactory) DefaultMessageExchangeFactory
                .createForEndpoint(consumerEndpointMock, null, targetEndpointMock);
        PetalsMessageExchange messageExchangeImpl = abstractMessageExchangeFactory.createExchange();

        assertEquals(consumerEndpointMock, messageExchangeImpl.getConsumerEndpoint());
        assertEquals(targetEndpointMock, messageExchangeImpl.getEndpoint());
        assertIdIsPetalsQualifiedUUID(messageExchangeImpl.getExchangeId());
    }
    
    @Test
    public void testBuildUseBuilderConsumerEpForInterface() throws MessagingException {
        QName targetInterface = new QName("http://test","interface");
        AbstractMessageExchangeFactory abstractMessageExchangeFactory = (AbstractMessageExchangeFactory) DefaultMessageExchangeFactory
                .createForInterface(consumerEndpointMock, null, targetInterface);
        PetalsMessageExchange messageExchangeImpl = abstractMessageExchangeFactory.createExchange();

        assertEquals(consumerEndpointMock, messageExchangeImpl.getConsumerEndpoint());
        assertEquals(targetInterface, messageExchangeImpl.getInterfaceName());
    }
    
    @Test
    public void testBuildUseBuilderConsumerEpForService() throws MessagingException {
        QName targetService = new QName("http://test","service");
        AbstractMessageExchangeFactory abstractMessageExchangeFactory = (AbstractMessageExchangeFactory) DefaultMessageExchangeFactory
                .createForService(consumerEndpointMock, null, targetService);
        PetalsMessageExchange messageExchangeImpl = abstractMessageExchangeFactory.createExchange();

        assertEquals(consumerEndpointMock, messageExchangeImpl.getConsumerEndpoint());
        assertEquals(targetService, messageExchangeImpl.getService());
    }
}
